package GCPlugins::GCfilms::GCAniDB;

###################################################
#
#  Copyright 2005-2010 Christian Jodar
#  Copyright 2024 Kerenoc
#  Copyright 2025 Lee Lee
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
#
###################################################

use strict;
use utf8;

use GCPlugins::GCfilms::GCfilmsCommon;
use Time::Piece;

{
    package GCPlugins::GCfilms::GCPluginAniDB;

    use base qw(GCPlugins::GCfilms::GCfilmsPluginsBase);

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;

        $self->{inside}->{$tagname}++;

        return if ($self->{parsingEnded});

        if ($self->{parsingList}) #Process result list example http://anidb.net/perl-bin/animedb.pl?show=animelist&adb.search=angel
        {
            #If it is <title> tag, make it run once.
            if ($tagname eq 'link' && $attr->{rel} eq 'canonical' && $attr->{href} =~ m|/anime/(\d+)|)
            {
                $self->{itemIdx}++;
                ($self->{itemsList}[$self->{itemIdx}]->{url} = $attr->{href}) =~ s/.*anidb.net//;

            }
            elsif ($tagname eq 'td')
            {
                $self->{isTdLabelTitle} = 1 if ($attr->{'data-label'} eq 'Title');
                $self->{isYear} = 1 if ($attr->{class} eq 'date airdate');
            }
            elsif ($tagname eq 'a' and $self->{isTdLabelTitle})
            {
                if ($attr->{href} =~ m/\/anime\/[0-9]+/)
                {
                    $self->{isMovie} = 1;
                    $self->{isInfo} = 1;
                    $self->{itemIdx}++ if ($self->{itemIdx} < 0) || ($attr->{href} ne $self->{itemsList}[$self->{itemIdx}]->{url});
                    $self->{itemsList}[$self->{itemIdx}]->{url} = $attr->{href};
                }
            }
            elsif ($tagname eq 'span' and $self->{isTdLabelTitle})
            {
                if (defined $attr->{class} and $attr->{class} eq 'match')
                {
                    $self->{isMatchName} = 1; # Match Name found. Example:<span class="match">(アクダマドライブ)</span>
                }
            }
            elsif ($tagname eq 'h1')
            {
                $self->{insideHeadline} = 1;
            }
        }
        else #Process anime info example https://anidb.net/anime/2773
        {
            if ($tagname eq 'img')
            {
                if ($attr->{src} =~ m/\/images\/main\//)
                {
                    $self->{curInfo}->{image} = $attr->{src} if !$self->{curInfo}->{image};
                }
            }
            elsif ($tagname eq 'div')
            {
                if ($attr->{itemprop} && $attr->{itemprop} eq 'description')
                {
                    $self->{insideSynopsis} = 1;
                }
                elsif ($attr->{id} && $attr->{id} eq 'userstats')
                {
                    $self->{insideSynopsis} = 0;
                }
            }
            elsif ($tagname eq 'br/' && $self->{insideSynopsis})
            {
                $self->{curInfo}->{synopsis} .= "\n";
            }
            elsif ($tagname eq 'th')
            {
                $self->{isField} = 1 if $attr->{class} eq 'field';
            }
            elsif ($tagname eq 'label' && $attr->{itemprop} eq 'alternateName' && $self->{isOriginal})
            {
                $self->{isOriginal} = 2;
            }
            elsif ($tagname eq 'span' && $attr->{itemprop} && $attr->{itemprop} eq 'genre' && $self->{isGenre})
            {
                $self->{isGenre} = 2;
            }
            elsif ($tagname eq 'span' && defined $attr->{itemprop} && $attr->{itemprop} eq 'name' && $self->{isParsedirName} && $self->{isFoundDir} eq 0)
            {
                $self->{isParseDir} = 1;
            }
            elsif ($tagname eq 'span' && defined $attr->{itemprop} && $attr->{itemprop} eq 'numberOfEpisodes')
            {
                $self->{isEpisodes} = 1;
            }
            elsif ($tagname eq 'span' && defined $attr->{title} && $attr->{title} =~ m/^language:/ && $attr->{title} !~ m/english/ && ! $self->{curInfo}->{country})
            {
                my $country = $attr->{title};
                $country =~ s/language: *//;
                $country =~ s/ *\(.*\)//;
                $country = $self->{countries}->{$country} if $self->{countries}->{$country};
                $self->{curInfo}->{country} = $country;
            }
            elsif ($tagname eq 'td' && defined $attr->{class})
            {
                if ($attr->{class} eq 'credit' && $self->{isParseDirectorCredit} eq 0 && $self->{isFoundDir} eq 0) #<td class="credit"><a href="/creator/?credit=1">Direction:</a>
                {
                    $self->{isParseDirectorCredit} = 1;
                }
            }
            elsif ($tagname eq 'a' && $self->{isParseDirectorCredit} && $self->{isParseDirectorA} eq 0 && $self->{isFoundDir} eq 0)
            {
                $self->{isParseDirectorA} = 1; #If found <a href="/creator/?credit=1">Direction:</a>
            }
            $self->{curInfo}->{synopsis} .= ' ' if $self->{insideSynopsis};
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;

        if ($tagname eq 'td')
        {
            if ($self->{isTdLabelTitle}) # initial false, become true when tr, data-label="Title"
            {
                $self->{isTdLabelTitle}=0;
            }
        }

        $self->{inside}->{$tagname}--;
    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if ($self->{parsingEnded});

        $origtext =~ s/[\r\n]//g;
        $origtext =~ s/^\s*//;
        $origtext =~ s/\s*$//;

        return if ($origtext eq '');

        if ($self->{parsingList})
        {
            if ($self->{insideHeadline})
            {
                $self->{parsingEnded} = 1 if $origtext !~ m/Anime List - Results for:/;
                $self->{insideHeadline} = 0;
            }

            if ($self->{isMovie})
            {
                my $num = $self->{itemIdx} + 1;
                $num = sprintf ("%03s",$num); # fix 3 digit length number
                $self->{itemsList}[$self->{itemIdx}]->{format} = $num;
                $self->{itemsList}[$self->{itemIdx}]->{title} = $origtext
                    if ! $self->{itemsList}[$self->{itemIdx}]->{title};
                $self->{mtitleIdx} = $self->{itemIdx};
                $self->{isMovie} = 0;
                $self->{isInfo} = 1;
                return;
            }
            elsif ($self->{isYear})
            {
                #Validate Date format. Example: 04.07.2025
                if ($origtext =~ /(\d{2})\.(\d{2})\.(\d{4})/ )
                {
                    $self->{itemsList}[$self->{itemIdx}]->{date} = $self->displayDateFormat($origtext,"%d.%m.%Y");# if $origtext =~ m/^ [0-9]{4}(-[0-9]{4})? $/;
                }
                $self->{isYear} = 0;
            }
            if ($self->{isMatchName})
            {
                $self->{isMatchName} = 0;
                my $num = $self->{mtitleIdx} + 1;
                $num = sprintf ("%03s",$num); # fix 3 digit length number
                $origtext =~ s/\(//g;    #Remove left Parentheses
                $origtext =~ s/\)//g;    #Remove right Parentheses
                $origtext = $num.") ".$origtext;    #Add sequencial numbering
                $self->{itemsList}[$self->{mtitleIdx}]->{title} = $origtext;
            }
        }
        else
        {
            if ($self->{isParseDirectorA} && $self->{isFoundDir} eq 0)
            {
                if  ($origtext eq 'Direction:')
                {
                    $self->{isParsedirName} = 1;
                }
            }
            if ($self->{isParseDir})
            {
                $self->{isParseDir} = 0;
                $self->{curInfo}->{director} = $origtext;
                $self->{isFoundDir} = 1;
                $self->{isParsedirName} = 0;
            }
            if ($self->{insideSynopsis})
            {
                $origtext =~ s/\s{2,}/ /g;
                $self->{curInfo}->{synopsis} .= $origtext;
            }
            elsif ($self->{isField})
            {
                $self->{isOriginal} = 0;
                $self->{isGenre} = 0;
                $self->{isTitle} = 1 if $origtext eq 'Main Title';
                $self->{isOriginal} = 1 if $origtext eq 'Official Title';
                $self->{isOrig} = 1 if $origtext =~ /kanji/i;
                $self->{isYear} = 1 if $origtext eq 'Year';
                $self->{isGenre} = 1 if $origtext eq 'Tags';
                $self->{isRating} = 1 if $origtext eq 'Rating';
                $self->{isType} = 1 if $origtext eq 'Type';
                $self->{isField} = 0;
            }
            elsif (1 || $self->{inside}->{td})
            {
                if ($self->{isTitle})
                {
                    $self->{curInfo}->{title} = $origtext;
                    $self->{isTitle} = 0;
                }
                elsif ($self->{isOriginal} eq 2)
                {
                    return if grep(/$origtext/, $self->{curInfo}->{original});
                    $self->{curInfo}->{original} .= " - " if $self->{curInfo}->{original};
                    $self->{curInfo}->{original} .= $origtext;
                    $self->{isOriginal} = 1;
                }
                elsif ($self->{isYear})
                {
                    if ($self->{serie})
                    {
                        $self->{curInfo}->{serie} = getMtype($self->{serie});
                        $self->{serie} = '';
                    }
                    if ($origtext =~ /([0-9][0-9])\.([0-9][0-9])\.([0-9][0-9][0-9][0-9])/)
                    {
                        $self->{curInfo}->{date} = "$1/$2/$3";
                    }
                    else
                    {
                        $self->{curInfo}->{date} = $origtext;
                    }
                    $self->{isYear} = 0;
                }
                elsif ($self->{isGenre} eq 2)
                {
                    $self->{curInfo}->{genre} .= ", " if $self->{curInfo}->{genre};
                    $self->{curInfo}->{genre} .= $origtext;
                    $self->{isGenre} = 1;
                }
                elsif ($self->{isRating})
                {
                    ($self->{curInfo}->{ratingpress} = $origtext) =~ s/\s//g;
                    $self->{isRating} = 0;
                }
                elsif ($self->{isType} && !$self->{curInfo}->{serie})
                {
                    $self->{serie} = $origtext;
                    $self->{isType} = 0;
                }
                elsif ($self->{isEpisodes})
                {
                    $self->{serie} .= $origtext;
                    $self->{isEpisodes} = 0;
                }
            }
        }
    }

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{hasField} = {
            format => 1,
            title => 1,
            date => 1,
            director => 0,
            actors => 0,
        };

        $self->{countries} = {
            japanese => "Japan",
            korean   => "Korea",
            chinese  => "China",
        };
        return $self;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        $self->{parsingEnded} = 0;
        $self->{isField} = 0;
        $self->{isInfo} = 0;
        $self->{isMovie} = 0;
        $self->{isGenre} = 0;
        $self->{isTitle} = 0;
        $self->{isOriginal} = 0;
        $self->{isRating} = 0;
        $self->{isYear} = 0;
        $self->{isType} = 0;
        $self->{isEpisodes} = 0;
        $self->{curName} = undef;
        $self->{curUrl} = undef;
        $self->{isTdLabelTitle} = 0;
        $self->{isOnlyRec} = 0; #Match only one result match and search result is auto redirected to the match page, such as http://anidb.net/perl-bin/animedb.pl?show=animelist&adb.search=6+Heart+Princess
        $self->{isMatchName} = 0; #If matched name exist. Search with non-alphebetic character such as Chinese, Japanese, Korean...
        $self->{mtitleIdx} = 0;
        $self->{serie} = '';

        $self->{isParseDirectorCredit} = 0; #Parse elements in <tr class="syn">
        $self->{isParseDirectorA} = 0; #Parse elements in <a href="/creator/?credit=1">Direction:</a>
        $self->{isParsedirName} = 0; #Parse elements in <a href="/creator/?credit=1">Direction:</a>
        $self->{isParseDir} = 0;
        $self->{isFoundDir} = 0;

        return $html;
    }

    sub getSearchUrl
    {
        my ($self, $word) = @_;

        # return "http://anidb.net/perl-bin/animedb.pl?xuser=$usr&xpass=$pas&show=animelist&adb.search=$word";
        return "http://anidb.net/perl-bin/animedb.pl?show=animelist&adb.search=$word";
    }

    sub getItemUrl
    {
        my ($self, $url) = @_;

        # return 'http://anidb.net' . $url . "?xuser=$usr&xpass=$pas";
        return 'http://anidb.net' . $url . '?';
    }

    sub getName
    {
        return 'AniDB';
    }

    sub getAuthor
    {
        return 'MeV - Lee Lee';
    }

    sub getLang
    {
        return 'EN';
    }

    sub getCharset
    {
        my $self = shift;

        return "UTF-8";
    }

    # Format UI diplay Date.
    # Return Date String in Year.Month.Day format
    # Check https://www.unix.com/man_page/freebsd/3/strftime/
    sub displayDateFormat
    {
        my $self = shift;
        my ($inputDate, $DateDesc) = @_;
        my $dispDate = Time::Piece->strptime($inputDate, $DateDesc);
        return $dispDate->strftime("%Y-%m-%d"); #Year.Month.Day
    }

    #Extract Type
    sub getMtype {
        my ($t) = @_;

        my $ret = "";
        my $toteps = "unknown";
        my $mType = "Other";

        if ($t =~ m/unknown/s)
        {
            if (!($t =~ m/(\d+)/i)) #If found number
            {
                $mType = $1 if ($t =~ m/(.*),/i); #If found series name
            }
            else
            {
                $toteps = $1;
            }
        }
        elsif ($t =~ m/(\d+)/i) #Find number. Example OVA, 4 episodes
        {
            $toteps = $1;
            $mType = $1 if ($t =~ m/(.*),/i);
        }
        else
        {
            $toteps = 1;
            $mType = $t;
        }
        $ret = $mType.": ".$toteps." eps";

        return ($ret);
    }
}

1;
